import copy

from django.conf import settings


DEFAULT_SETTINGS = {
    'cache_prefix': 'LOCATION',
    'runmeter_mailbox': None,
    'icloud': {
        'min_horizontal_accuracy': 20,
        'max_wait_seconds': 120,
        'request_interval_seconds': 5,
    },
    'periodic_consumers': [
        'location.consumers.runmeter.RunmeterConsumer',
        'location.consumers.icloud.iCloudConsumer',
    ]
}

SETTINGS = copy.deepcopy(DEFAULT_SETTINGS)
SETTINGS.update(
    getattr(
        settings,
        'DJANGO_LOCATION_SETTINGS',
        {}
    )
)



from django.dispatch.dispatcher import Signal

from location.models import LocationSnapshot


location_updated = Signal(providing_args=['user', 'from_', 'to'])
location_changed = Signal(providing_args=['user', 'from_', 'to'])


class watch_location(object):
    def __init__(self, user):
        self.user = user

    def _get_current_location(self):
        return LocationSnapshot.objects.filter(
            source__user=self.user,
        ).order_by('-date')[0]

    def __enter__(self):
        self.original_location = None
        try:
            self.original_location = self._get_current_location()
        except IndexError:
            pass
        return self

    def __exit__(self, *args):
        current_location = self._get_current_location()
        if self.original_location != current_location:
            location_updated.send(
                sender=self,
                user=self.user,
                from_=self.original_location,
                to=current_location,
            )
            if (
                self.original_location and
                self.original_location.location
                != current_location.location
            ):
                location_changed.send(
                    sender=self,
                    user=self.user,
                    from_=self.original_location,
                    to=current_location,
                )



import datetime

from django.contrib.gis.geos import Point
from django.dispatch import receiver
from django.utils.timezone import utc

from location.models import (
    LocationSnapshot,
    LocationSource,
    LocationSourceType,
)
from location.signals import location_updated, watch_location
from location.tests.base import BaseTestCase


class SignalTest(BaseTestCase):
    def setUp(self):
        super(SignalTest, self).setUp()
        self.signal_receipts = []

        self.source_type, _ = LocationSourceType.objects.get_or_create(
            name='Arbitrary Source Type'
        )
        self.source = LocationSource.objects.create(
            name='Arbitrary Source',
            type=self.source_type,
            user=self.user,
            active=False,
        )
        self.snapshot = LocationSnapshot.objects.create(
            source=self.source,
            location=Point(
                10,
                10
            ),
            date=datetime.datetime.utcnow().replace(tzinfo=utc)
        )

    def test_watch_location_changed(self):
        @receiver(location_updated, dispatch_uid='signal_test_uid')
        def process_incoming_location(*args, **kwargs):
            self.signal_receipts.append({
                'args': args,
                'kwargs': kwargs,
            })

        with watch_location(self.user):
            LocationSnapshot.objects.create(
                source=self.source,
                location=Point(
                    10,
                    11
                ),
                date=datetime.datetime.utcnow().replace(tzinfo=utc)
            )

        self.assertTrue(
            len(self.signal_receipts) == 1
        )

    def test_watch_location_unchanged(self):
        @receiver(location_updated, dispatch_uid='signal_test_uid')
        def process_incoming_location(*args, **kwargs):
            self.signal_receipts.append({
                'args': args,
                'kwargs': kwargs,
            })

        with watch_location(self.user):
            pass

        self.assertTrue(
            len(self.signal_receipts) == 0
        )

    def test_watch_location_old(self):
        @receiver(location_updated, dispatch_uid='signal_test_uid')
        def process_incoming_location(*args, **kwargs):
            self.signal_receipts.append({
                'args': args,
                'kwargs': kwargs,
            })

        with watch_location(self.user):
            LocationSnapshot.objects.create(
                source=self.source,
                location=Point(
                    10,
                    11
                ),
                date=(
                    datetime.datetime.utcnow().replace(tzinfo=utc)
                    - datetime.timedelta(hours=1)
                )
            )

        self.assertTrue(
            len(self.signal_receipts) == 0
        )



#!/usr/bin/env python
import os
import sys

from os.path import dirname, abspath

from django.conf import settings

TRAVIS = False
if os.environ.get('TRAVIS') is not None:
    TRAVIS = True

if not settings.configured:
    DATABASES = {
        'default': {
            'ENGINE': 'django.contrib.gis.db.backends.spatialite',
            'NAME': ':memory:'
        },
    }
    if TRAVIS:
        DATABASES = {
            'default': {
                'ENGINE': 'django.contrib.gis.db.backends.postgis',
                'NAME': 'django_location',
                'USERNAME': 'postgres',
                'HOST': '127.0.0.1'
            }
        }
    settings.configure(
        POSTGIS_VERSION=(1, 5, 3),
        DATABASES=DATABASES,
        INSTALLED_APPS=[
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django_mailbox',
            'location',
        ],
        USE_TZ=True,
    )

from django.test.simple import DjangoTestSuiteRunner


def runtests(*test_args):
    if not test_args:
        test_args = ['location']
    parent = dirname(abspath(__file__))
    sys.path.insert(0, parent)

    if not TRAVIS:
        from django.db import connection
        cursor = connection.cursor()
        cursor.execute("SELECT InitSpatialMetaData();")

    runner = DjangoTestSuiteRunner(
        verbosity=1,
        interactive=False,
        failfast=False
    )
    failures = runner.run_tests(test_args)
    sys.exit(failures)


if __name__ == '__main__':
    runtests(*sys.argv[1:])



import logging
from optparse import make_option

from django.core.management.base import BaseCommand
from django.db import transaction

from location.settings import SETTINGS


logger = logging.getLogger(__name__)


def get_class_by_path(path):
    mod = __import__('.'.join(path.split('.')[:-1]))
    components = path.split('.')
    for comp in components[1:]:
        mod = getattr(mod, comp)
    return mod


logger = logging.getLogger(__name__)


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option(
            '--loglevel',
            default=None,
        ),
    )

    @transaction.commit_on_success
    def handle(self, *args, **options):
        # Only set logging if it isn't already configured
        if options['loglevel'] is not None:
            logging.basicConfig(
                level=logging.getLevelName(options['loglevel'])
            )

        for consumer_path in SETTINGS['periodic_consumers']:
            logger.info("Running periodic consumer '%s'.", consumer_path)
            try:
                consumer_cls = get_class_by_path(consumer_path)
                consumer_cls.periodic()
            except ImportError:
                logger.exception('Unable to import consumer.')
            except:
                logger.exception('Error encountered while executing consumer.')



from django.contrib.auth.models import User
from django.test import TestCase


class BaseTestCase(TestCase):
    def setUp(self):
        self.user = User.objects.create(
            username='arbitrary_username',
        )



import calendar
import datetime

from django.contrib.gis.geos import Point
from django.utils.timezone import utc
from mock import MagicMock, patch
import pyicloud

from location import models
from location.tests.base import BaseTestCase
from location.consumers import icloud


class iCloudTest(BaseTestCase):
    def setUp(self):
        super(iCloudTest, self).setUp()
        self.arbitrary_username = 'arbitrary_username'
        self.arbitrary_password = 'arbitrary_password'
        self.arbitrary_device_id = 'arbitrary_device_id'
        self.user_settings = models.LocationConsumerSettings.objects.create(
            user=self.user,
            icloud_enabled=True,
            icloud_username=self.arbitrary_username,
            icloud_password=self.arbitrary_password,
            icloud_device_id=self.arbitrary_device_id,
            icloud_timezone='UTC'
        )
        icloud.SETTINGS['icloud']['max_wait_seconds'] = 0.5
        icloud.SETTINGS['icloud']['request_interval_seconds'] = 0.1
        self.icloud_consumer = icloud.iCloudConsumer(self.user_settings)

    def test_get_location_data_unknown_device_id(self):
        with patch('pyicloud.PyiCloudService.__init__') as init_mock:
            init_mock.return_value = None

            pyicloud.PyiCloudService.devices = {}

            with self.assertRaises(icloud.UnknownDeviceException):
                self.icloud_consumer.get_location_data()

    def test_get_location_data(self):
        arbitrary_location_data = {
            'somewhere': 'around',
            'here': True
        }
        with patch('pyicloud.PyiCloudService.__init__') as init_mock:
            init_mock.return_value = None
            mock_device = MagicMock()
            mock_device.location.return_value = arbitrary_location_data
            pyicloud.PyiCloudService.devices = {}
            pyicloud.PyiCloudService.devices[self.arbitrary_device_id] = (
                mock_device
            )
            self.icloud_consumer.data_is_accurate = MagicMock()
            self.icloud_consumer.data_is_accurate.return_value = True

            actual_data = self.icloud_consumer.get_location_data()

            mock_device.location.assert_called_with()

            self.assertEqual(
                actual_data,
                arbitrary_location_data
            )

    def test_get_location_data_inaccurate(self):
        arbitrary_location_data = {
            'somewhere': 'around',
            'here': True
        }
        with patch('pyicloud.PyiCloudService.__init__') as init_mock:
            init_mock.return_value = None
            mock_device = MagicMock()
            mock_device.location.return_value = arbitrary_location_data
            pyicloud.PyiCloudService.devices = {}
            pyicloud.PyiCloudService.devices[self.arbitrary_device_id] = (
                mock_device
            )
            self.icloud_consumer.data_is_accurate = MagicMock()
            self.icloud_consumer.data_is_accurate.return_value = False

            with self.assertRaises(icloud.LocationUnavailableException):
                self.icloud_consumer.get_location_data()

    def test_data_is_accurate(self):
        accurate_data = {
            'locationFinished': True,
            'isInaccurate': False,
            'isOld': False,
            'horizontalAccuracy': 1,
        }

        actual_result = self.icloud_consumer.data_is_accurate(accurate_data)

        self.assertTrue(
            actual_result,
        )

    def test_location_not_finished(self):
        failure_data = {
            'locationFinished': False,
            'isInaccurate': False,
            'isOld': False,
            'horizontalAccuracy': 1,
        }

        actual_result = self.icloud_consumer.data_is_accurate(failure_data)

        self.assertFalse(
            actual_result,
        )

    def test_location_inaccurate(self):
        failure_data = {
            'locationFinished': True,
            'isInaccurate': True,
            'isOld': False,
            'horizontalAccuracy': 1,
        }

        actual_result = self.icloud_consumer.data_is_accurate(failure_data)

        self.assertFalse(
            actual_result,
        )

    def test_location_is_old(self):
        failure_data = {
            'locationFinished': True,
            'isInaccurate': False,
            'isOld': True,
            'horizontalAccuracy': 1,
        }

        actual_result = self.icloud_consumer.data_is_accurate(failure_data)

        self.assertFalse(
            actual_result,
        )

    def test_insufficient_horizontal_accuracy(self):
        failure_data = {
            'locationFinished': True,
            'isInaccurate': False,
            'isOld': True,
            'horizontalAccuracy': (
                icloud.SETTINGS['icloud']['min_horizontal_accuracy'] * 2
            ),
        }

        actual_result = self.icloud_consumer.data_is_accurate(failure_data)

        self.assertFalse(
            actual_result,
        )

    def test_update_location(self):
        arbitrary_time = datetime.datetime(2013, 3, 2).replace(tzinfo=utc)
        arbitrary_latitude = 50
        arbitrary_longitude = 75
        mock_location_data = {
            'timeStamp': calendar.timegm(arbitrary_time.timetuple()) * 1000,
            'longitude': arbitrary_longitude,
            'latitude': arbitrary_latitude,
        }

        self.icloud_consumer.get_location_data = MagicMock()
        self.icloud_consumer.get_location_data.return_value = (
            mock_location_data
        )

        self.icloud_consumer.update_location()

        snapshot = models.LocationSnapshot.objects.get()
        self.assertIsNotNone(snapshot.source)
        self.assertEqual(
            snapshot.location,
            Point(arbitrary_longitude, arbitrary_latitude)
        )
        self.assertEqual(
            snapshot.date,
            arbitrary_time,
        )



import logging

from django.conf.urls import patterns, url
from django.contrib.gis import admin
from django.contrib.messages.api import get_messages
from django.contrib.sites.models import Site
from django.template.response import TemplateResponse

from location.models import (
    LocationConsumerSettings,
    LocationSnapshot,
    LocationSource,
    LocationSourceType
)

logger = logging.getLogger('location.admin')


class LocationSourceAdmin(admin.options.OSMGeoAdmin):
    list_display = (
        'created',
        'name',
        'user',
        'type',
        'active'
    )
    list_filter = [
        'type'
    ]
    ordering = ['-created']
    raw_id_fields = ('user', )

    def get_urls(self):
        urls = super(LocationSourceAdmin, self).get_urls()

        urls = patterns(
            'location.views',
            url(
                'configure-accounts/',
                self.admin_site.admin_view(self.configure_accounts),
                name='location_configure_accounts'
            )
        ) + urls
        return urls

    def configure_accounts(self, request):
        logger.info(self.model._meta)
        logger.info(self.model._meta.app_label)
        return TemplateResponse(
            request,
            'admin/location/configure.html', {
                'messages': get_messages(request),
                'title': 'Configure Accounts',
                'domain': Site.objects.get_current().domain
            }
        )


class LocationSnapshotAdmin(admin.options.OSMGeoAdmin):
    list_display = (
        'date',
        'neighborhood',
        'nearest_city',
    )
    date_hierarchy = 'date'
    raw_id_fields = ('source', )
    list_per_page = 25
    ordering = ['-date']
    list_filter = [
        'source__type'
    ]
    search_fields = [
        'source__type',
        'source__user__username',
    ]

    def nearest_city(self, obj):
        city = obj.city
        if city:
            return city
        nearest = obj.find_nearest_city()
        if nearest.distance is not None:
            return "%s (%d mi away)" % (
                nearest,
                nearest.distance.mi
            )
        return nearest


class LocationConsumerSettingsAdmin(admin.options.OSMGeoAdmin):
    raw_id_fields = ('user', )


admin.site.register(LocationSourceType)
admin.site.register(LocationSource, LocationSourceAdmin)
admin.site.register(LocationSnapshot, LocationSnapshotAdmin)
admin.site.register(LocationConsumerSettings, LocationConsumerSettingsAdmin)



import datetime
import logging

from django.conf import settings
from django.contrib.gis.db import models
from django.core.cache import cache
from django.dispatch import receiver
from django_mailbox.signals import message_received
from jsonfield.fields import JSONField

from location.settings import SETTINGS


logger = logging.getLogger('location.models')


try:
    from census_places.models import PlaceBoundary
except ImportError:
    logger.warning(
        "django-census-places is not installed, locations will not "
        "be populated with city information."
    )
    PlaceBoundary = None
try:
    from neighborhoods.models import Neighborhood
except ImportError:
    logger.warning(
        "django-neighborhoods is not installed, locations will not "
        "be populated with neighborhood information."
    )
    Neighborhood = None


class LocationConsumerSettings(models.Model):
    user = models.OneToOneField(
        getattr(
            settings,
            'AUTH_USER_MODEL',
            'auth.User'
        ),
        related_name='location_consumer_settings'
    )
    icloud_enabled = models.BooleanField(default=False)
    icloud_timezone = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        default='US/Pacific',
    )
    icloud_username = models.CharField(
        max_length=255,
        blank=True,
        null=True,
    )
    icloud_password = models.CharField(
        max_length=255,
        blank=True,
        null=True,
    )
    icloud_device_id = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        help_text=(
            "Device ID of the iCloud device from which to gather periodic"
            "location updates"
        )
    )
    runmeter_enabled = models.BooleanField(default=False)
    runmeter_email = models.EmailField(
        max_length=255,
        help_text=(
            "E-mail address of the device from which RunMeter will be sending"
            "location updates"
        )
    )

    def __unicode__(self):
        return "Location Consumer Settings for %s" % (
            self.user.get_username()
        )

    class Meta:
        verbose_name = 'Location Consumer Settings'
        verbose_name_plural = 'Location Consumer Settings'


class LocationSourceType(models.Model):
    name = models.CharField(max_length=255)
    icon = models.ImageField(
        null=True,
        blank=True,
        upload_to='source_type_icons/'
    )

    def __unicode__(self):
        return self.name


class LocationSource(models.Model):
    name = models.CharField(max_length=255)
    user = models.ForeignKey(
        getattr(
            settings,
            'AUTH_USER_MODEL',
            'auth.User'
        ),
        related_name='location_sources',
        null=True,
        default=None,
    )
    type = models.ForeignKey(LocationSourceType)
    data = JSONField()
    created = models.DateTimeField(
        auto_now_add=True
    )
    updated = models.DateTimeField(
        auto_now=True
    )
    active = models.BooleanField(
        default=False
    )

    def __unicode__(self):
        return "%s: %s" % (
            self.type.name,
            self.name,
        )


class LocationSnapshot(models.Model):
    location = models.PointField(
        geography=True,
        spatial_index=True
    )
    source = models.ForeignKey(
        LocationSource,
        related_name='points',
        null=True,
        blank=True
    )
    date = models.DateTimeField(
        default=datetime.datetime.now
    )

    created = models.DateTimeField(
        auto_now_add=True
    )

    objects = models.GeoManager()

    def get_cache_key(self, name):
        return '%s:%s:%s:%s' % (
            SETTINGS['cache_prefix'],
            self.__class__.__name__,
            self.pk,
            name
        )

    def get_cached(self, name):
        return cache.get(self.get_cache_key(name))

    def set_cached(self, name, value):
        cache.set(self.get_cache_key(name), value, 60 * 60 * 24)

    @property
    def city(self):
        if PlaceBoundary:
            cached = self.get_cached('city')
            if cached:
                return cached
            try:
                result = PlaceBoundary.get_containing(self.location)
                self.set_cached('city', result)
                return result
            except PlaceBoundary.DoesNotExist:
                pass
        return None

    @property
    def neighborhood(self):
        if Neighborhood:
            cached = self.get_cached('neighborhood')
            if cached:
                return cached
            try:
                result = Neighborhood.get_containing(self.location)
                self.set_cached('neighborhood', result)
                return result
            except Neighborhood.DoesNotExist:
                pass
        return None

    def find_nearest_city(self):
        if PlaceBoundary:
            cached = self.get_cached('nearest_city')
            if cached:
                return cached
            try:
                result = PlaceBoundary.get_nearest_to(self.location)
                self.set_cached('nearest_city', result)
                return result
            except PlaceBoundary.DoesNotExist:
                pass
        return None

    def __unicode__(self):
        return u"%s's location at %s" % (
            self.source.user,
            self.date
        )


@receiver(message_received, dispatch_uid='process_incoming_runmeter_msg')
def process_incoming_runmeter_message(sender, message, **kwargs):
    from location.consumers.runmeter import RunmeterConsumer
    if message.mailbox.name == SETTINGS['runmeter_mailbox']:
        try:
            RunmeterConsumer.process_message(message)
        except LocationConsumerSettings.DoesNotExist:
            logger.warning(
                'Unable to process message \'%s\': '
                'No user is currently assigned to from_address %s',
                message.from_address
            )



from django.core.management.base import BaseCommand
import pyicloud


class Command(BaseCommand):
    args = '<apple icloud username> <apple icloud password>'
    help = 'List devices associated with this iCloud account'

    def handle(self, *args, **kwargs):
        icloud_username = args[0]
        icloud_password = args[1]

        api = pyicloud.PyiCloudService(icloud_username, icloud_password)

        for id, device in api.devices.items():
            print 'Name: %s -- ID: %s' % (
                str(device),
                str(id),
            )



from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt

from location.consumers.foursquare import FoursquareConsumer


@csrf_exempt
def foursquare_checkin(request):
    consumer = FoursquareConsumer(
        request.POST.get('checkin', None)
    )
    consumer.process_checkin()
    return HttpResponse("OK")



import datetime
import logging
import re

from django.contrib.gis.geos import Point
from django.utils.timezone import utc
from django.db.models import Max
from lxml import objectify
import requests
from requests.adapters import HTTPAdapter

from location.models import (
    LocationConsumerSettings,
    LocationSnapshot,
    LocationSource,
    LocationSourceType
)
from location.signals import watch_location


logger = logging.getLogger(__name__)


class RunmeterConsumer(object):
    NAMESPACES = {
        'kml': 'http://www.opengis.net/kml/2.2',
        'abvio': 'http://www.abvio.com/xmlschemas/1'
    }

    def __init__(self, source):
        self.source = source
        self.session = requests.Session()
        self.session.mount(
            'http://',
            HTTPAdapter(max_retries=5),
        )
        self.session.mount(
            'https://',
            HTTPAdapter(max_retries=5),
        )

    @classmethod
    def periodic(cls):
        cls.process_active_sources()

    @classmethod
    def process_message(cls, message):
        settings = LocationConsumerSettings.objects.get(
            runmeter_enabled=True,
            runmeter_email=message.from_address[0],
        )

        url = cls.get_import_url_from_message_body(message.text)
        source = cls.get_source_from_user_and_url(settings.user, url)

        if cls.message_indicates_finish(message.text):
            source.active = False

        instance = RunmeterConsumer(source)
        instance.process()

        message.read = datetime.datetime.utcnow().replace(tzinfo=utc)
        message.save()

        return instance

    @classmethod
    def process_active_sources(cls):
        source_type = cls.get_source_type()
        sources = LocationSource.objects.filter(
            type=source_type,
            active=True,
        )
        for source in sources:
            logger.debug(
                'Found active source %s.', source
            )
            instance = RunmeterConsumer(source)
            instance.process()

    def process(self):
        logger.info('Processing source %s.', self.source)
        document = self._get_document(self.source.data['url'])

        base_time = self.get_start_time(document)
        route_name = self.get_route_name(document)

        with watch_location(self.source.user):
            for raw_point in self.get_points(document, base_time):
                key_name = raw_point['key']
                if isinstance(self.source.data['known_points'], list):
                    self.source.data['known_points'] = {}
                if key_name not in self.source.data['known_points']:
                    point = self._get_processed_point(raw_point, base_time)
                    logger.debug(
                        'Creating point %s,%s at %s.',
                        point['lat'],
                        point['lng'],
                        point['date'],
                    )
                    LocationSnapshot.objects.create(
                        source=self.source,
                        location=point['point'],
                        date=point['date'],
                    )
                    self.source.data['known_points'][key_name] = raw_point
                else:
                    logger.debug(
                        'Point %s,%s already stored.',
                        raw_point['lat'],
                        raw_point['lng'],
                    )

        if route_name:
            self.source.name = '%s (%s)' % (
                route_name if route_name else 'AdHoc',
                self.source.data['url']
            )
        if self.source.active:
            self.source.active = self.is_active()
            if not self.source.active:
                logger.debug('Source has expired; marked inactive.')
        self.source.save()

    def get_route_name(self, document):
        values = document.xpath(
            '//abvio:routeName',
            namespaces=self.NAMESPACES
        )
        if values:
            return values[0].text
        values = document.xpath(
            '//abvio:activityName',
            namespaces=self.NAMESPACES
        )
        if values:
            return values[0].text
        return None

    def get_start_time(self, document):
        start_string = document.xpath(
            '//abvio:startTime',
            namespaces=self.NAMESPACES
        )[0].text

        return datetime.datetime.strptime(
            start_string[0:19],
            "%Y-%m-%d %H:%M:%S"
        ).replace(
            tzinfo=utc
        )

    def get_points(self, document, base_time):
        points = []
        coordinate_table = document.xpath(
            '//abvio:coordinateTable',
            namespaces=self.NAMESPACES
        )[0].text.split('\n')
        for coordinate_row in coordinate_table:
            if coordinate_row:
                cols = coordinate_row.split(',')
                points.append({
                    'key': coordinate_row,
                    'time': float(cols[0]),
                    'lng': float(cols[1]),
                    'lat': float(cols[2]),
                })
        return points

    def _get_processed_point(self, point, base_time):
        point = point.copy()
        point['point'] = Point(
            point['lat'],
            point['lng'],
        )
        point['date'] = base_time + datetime.timedelta(
            seconds=point['time']
        )
        return point

    def _get_document(self, url):
        return objectify.fromstring(
            self.session.get(url, timeout=10.0).content
        )

    @classmethod
    def get_source_type(cls):
        source_type, _ = LocationSourceType.objects.get_or_create(
            name='Runmeter'
        )
        return source_type

    @classmethod
    def get_source_from_user_and_url(cls, user, url, **additional_filters):
        source_type = cls.get_source_type()
        source = None
        now = datetime.datetime.utcnow().replace(tzinfo=utc)
        minimum_time = now - datetime.timedelta(days=1)
        for potential_source in LocationSource.objects.filter(
            created__gt=minimum_time,
            user=user,
            type=source_type,
            **additional_filters
        ):
            data = potential_source.data
            if 'url' in data.keys() and data['url'] == url:
                source = potential_source
                logger.info("Found existing source with ID %s.", source.id)
                break
        if not source:
            source = LocationSource.objects.create(
                type=source_type,
                name="Runmeter Route at %s" % datetime.datetime.now(),
                data={
                    'url': url,
                    'known_points': {}
                },
                user=user,
                active=True,
            )
            logger.info("Created new source with ID %s.", source.id)
        return source

    def is_active(self):
        max_date = self.source.points.aggregate(avg=Max('date'))['avg']
        now = datetime.datetime.utcnow().replace(tzinfo=utc)
        logger.debug("Max Date %s", max_date)
        logger.debug("Now %s", now)
        if max_date and now - max_date > datetime.timedelta(minutes=60):
            return False
        return True

    @classmethod
    def message_indicates_finish(self, body):
        if re.search('Finished [A-Za-z0-9-_]+:', body):
            return True
        return False

    @classmethod
    def get_import_url_from_message_body(self, body):
        matches = re.search(r"Import Link: (.*)", body)
        if not matches:
            matches = re.search(r"Import URL: (.*)", body)
            if not matches:
                return None
        return matches.groups()[0].strip()



import datetime
import logging
import time

from django.contrib.gis.geos import Point
import pyicloud
import pytz

from location.models import (
    LocationConsumerSettings,
    LocationSnapshot,
    LocationSource,
    LocationSourceType
)
from location.settings import SETTINGS
from location.signals import watch_location


logger = logging.getLogger(__name__)


class LocationUnavailableException(Exception):
    pass


class UnknownDeviceException(Exception):
    pass


class iCloudConsumer(object):
    def __init__(self, user_settings):
        self.user_settings = user_settings

    @classmethod
    def periodic(cls):
        users = cls.get_icloud_enabled_settings()
        for user_settings in users:
            instance = cls(user_settings)
            try:
                instance.update_location()
            except pyicloud.exceptions.PyiCloudFailedLoginException:
                logger.exception(
                    'Unable to log-in to iCloud with the provided credentials '
                    '; disabling icloud for %s',
                    user_settings,
                )
                cls.disable_icloud(
                    user_settings,
                    "Unable to login to iCloud account '%s' using provided "
                    "credentials." % (
                        user_settings.icloud_device_id,
                        user_settings.icloud_username,
                    )
                )
            except UnknownDeviceException:
                logger.exception(
                    'Unable to find device on the account using the provided '
                    'credentials',
                    user_settings,
                )
                cls.disable_icloud(
                    user_settings,
                    "Unable to find device '%s' on iCloud account '%s'." % (
                        user_settings.icloud_device_id,
                        user_settings.icloud_username,
                    )
                )
            except LocationUnavailableException as e:
                logger.warning(
                    'Location currently unavailable for consumer settings %s',
                    instance,
                )
            except Exception as e:
                logger.exception(
                    'Unable to gather iCloud location for location consumer'
                    'settings %s: %s',
                    instance,
                    e
                )

    @classmethod
    def disable_icloud(cls, user_settings, message=None):
        user_settings.icloud_enabled = False
        user_settings.save()
        if message is not None:
            user_settings.user.email_user(
                'iCloud location consumption disabled',
                message
            )

    @classmethod
    def get_icloud_enabled_settings(cls):
        return LocationConsumerSettings.objects.filter(
            icloud_enabled=True
        )

    def get_location_data(self):
        api = pyicloud.PyiCloudService(
            self.user_settings.icloud_username,
            self.user_settings.icloud_password,
        )

        started = time.time()
        while time.time() - started < SETTINGS['icloud']['max_wait_seconds']:
            try:
                device = api.devices[self.user_settings.icloud_device_id]
            except KeyError:
                raise UnknownDeviceException(
                    'Device %s not found.' % (
                        self.user_settings.icloud_device_id,
                    )
                )

            data = device.location()

            logger.debug(
                'Gathered data %s from device %s.',
                data,
                self.user_settings.icloud_device_id,
            )
            if self.data_is_accurate(data):
                return data
            time.sleep(SETTINGS['icloud']['request_interval_seconds'])

        raise LocationUnavailableException(
            'Unable to acquire location of device %s within %s seconds',
            api.devices[self.user_settings.icloud_device_id],
            SETTINGS['icloud']['max_wait_seconds'],
        )

    def data_is_accurate(self, data):
        if not data:
            logger.info("No location data available.")
            return False
        elif not data['locationFinished']:
            logger.info("Location not ready.")
            return False
        elif data['isInaccurate']:
            logger.info('Location marked as inaccurate')
            return False
        elif data['isOld']:
            logger.info('Location explicitly marked as old')
            return False
        elif (
            data['horizontalAccuracy'] > (
                SETTINGS['icloud']['min_horizontal_accuracy']
            )
        ):
            logger.info(
                'Horizontal accuracy insufficient (%s > %s)',
                data['horizontalAccuracy'],
                SETTINGS['icloud']['min_horizontal_accuracy']
            )
            return False
        return True

    def update_location(self):
        source_type = self.get_source_type()
        data = self.get_location_data()

        local_tz = pytz.timezone(self.user_settings.icloud_timezone)

        date = datetime.datetime.fromtimestamp(
            float(data['timeStamp']) / 1000,
            local_tz
        )

        if LocationSnapshot.objects.filter(
            date__gt=date - datetime.timedelta(minutes=1),
            source__type=source_type,
            source__user=self.user_settings.user,
        ).exists():
            logger.info(
                'Found another sample from within the last minute; skipping '
                'gathered icloud location'
            )
            return

        with watch_location(self.user_settings.user):
            source = LocationSource.objects.create(
                name='Apple iCloud location at %s' % date,
                type=source_type,
                user=self.user_settings.user,
                data=data,
                active=False,
            )
            return LocationSnapshot.objects.create(
                source=source,
                location=Point(
                    data['longitude'],
                    data['latitude'],
                ),
                date=date,
            )

    @classmethod
    def get_source_type(cls):
        source_type, _ = LocationSourceType.objects.get_or_create(
            name='Apple iCloud'
        )
        return source_type



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'LocationSource.user'
        db.add_column(u'location_locationsource', 'user',
                      self.gf('django.db.models.fields.related.ForeignKey')(default=None, related_name='location_sources', null=True, to=orm['auth.User']),
                      keep_default=False)

    def backwards(self, orm):
        # Deleting field 'LocationSource.user'
        db.delete_column(u'location_locationsource', 'user_id')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'location.locationconsumersettings': {
            'Meta': {'object_name': 'LocationConsumerSettings'},
            'icloud_device_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'icloud_enabled': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'icloud_password': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'icloud_username': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'runmeter_email': ('django.db.models.fields.EmailField', [], {'max_length': '255'}),
            'runmeter_enabled': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'user': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'location_consumer_settings'", 'unique': 'True', 'to': u"orm['auth.User']"})
        },
        u'location.locationsnapshot': {
            'Meta': {'object_name': 'LocationSnapshot'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'location': ('django.contrib.gis.db.models.fields.PointField', [], {'geography': 'True'}),
            'source': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'points'", 'null': 'True', 'to': u"orm['location.LocationSource']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'location_snapshots'", 'to': u"orm['auth.User']"})
        },
        u'location.locationsource': {
            'Meta': {'object_name': 'LocationSource'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'data': ('jsonfield.fields.JSONField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['location.LocationSourceType']"}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'related_name': "'location_sources'", 'null': 'True', 'to': u"orm['auth.User']"})
        },
        u'location.locationsourcetype': {
            'Meta': {'object_name': 'LocationSourceType'},
            'icon': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        }
    }

    complete_apps = ['location']



from setuptools import setup, find_packages

requirements = []
with open('requirements.txt', 'r') as in_:
    requirements = in_.readlines()

setup(
    name='django-location',
    version='2.1.1',
    url='http://github.com/latestrevision/django-location/',
    description='Gather, store, and display real-time location information from Foursquare, iCloud, and more.',
    author='Adam Coddington',
    author_email='me@adamcoddington.net',
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering :: GIS',
    ],
    include_package_data=True,
    extras_require={
        'locationdetails': [
            'django-neighborhoods',
            'django-census-places',
        ],
    },
    tests_require=[
        'mock>=1.0.1',
    ],
    test_suite='location.runtests.runtests',
    install_requires=requirements,
    packages=find_packages()
)



import calendar
import datetime
import json

from django.contrib.gis.geos import Point
from django.utils.timezone import utc
from mock import MagicMock

from location import models
from location.tests.base import BaseTestCase
from location.consumers import foursquare


class FoursquareTest(BaseTestCase):
    def test_process_checkin(self):
        arbitrary_latitude = 60
        arbitrary_longitude = 32
        arbitrary_date = datetime.datetime(2013, 3, 2).replace(tzinfo=utc)
        checkin_data = {
            'type': 'checkin',
            'venue': {
                'location': {
                    'lat': arbitrary_latitude,
                    'lng': arbitrary_longitude,
                },
                'name': 'Test Location',
            },
            'createdAt': calendar.timegm(arbitrary_date.timetuple()),
            'timeZone': 'UTC'
        }

        foursquare.FoursquareConsumer.get_user = MagicMock()
        foursquare.FoursquareConsumer.get_user.return_value = self.user

        self.foursquare_consumer = foursquare.FoursquareConsumer(
            json.dumps(checkin_data)
        )
        self.foursquare_consumer.process_checkin()

        location = models.LocationSnapshot.objects.get()
        self.assertEqual(
            location.location,
            Point(arbitrary_longitude, arbitrary_latitude)
        )
        self.assertEqual(
            location.date,
            arbitrary_date,
        )
        self.assertIsNotNone(location.source)



import datetime
import json
import os.path

from django.contrib.gis.geos import Point
from django.utils.timezone import utc
from django_mailbox.models import Mailbox, Message
from django_mailbox.signals import message_received
from lxml import objectify
from mock import MagicMock, patch

from location import models
from location.tests.base import BaseTestCase
from location.consumers.runmeter import RunmeterConsumer


class RunmeterTest(BaseTestCase):
    def setUp(self):
        super(RunmeterTest, self).setUp()
        self.arbitrary_email = 'phlogiston@becker.net'
        self.arbitrary_mailbox = 'Runmeter Mailbox'
        self.settings = models.LocationConsumerSettings.objects.create(
            user=self.user,
            runmeter_enabled=True,
            runmeter_email=self.arbitrary_email,
        )
        self.mailbox = Mailbox.objects.create(
            name=self.arbitrary_mailbox,
        )
        self.source_type = RunmeterConsumer.get_source_type()
        models.SETTINGS['runmeter_mailbox'] = self.arbitrary_mailbox

    def _get_sample_document(self):
        file_path = os.path.join(
            os.path.dirname(__file__),
            'files/sample_cycle.kml',
        )
        with open(file_path, 'r') as incoming:
            return objectify.fromstring(incoming.read())

    @patch.object(RunmeterConsumer, 'get_import_url_from_message_body')
    @patch.object(RunmeterConsumer, 'get_source_from_user_and_url')
    @patch.object(RunmeterConsumer, 'message_indicates_finish')
    @patch.object(RunmeterConsumer, 'process')
    def test_process_message(
        self, process, message_indicates_finish,
        get_source_from_user_and_url, get_import_url_from_message_body
    ):
        arbitrary_body = 'I am the very model of a modern major general'
        arbitrary_url = 'http://go.com/'
        arbitrary_source = models.LocationSource.objects.create(
            name='Whatnot',
            user=self.user,
            type=self.source_type,
        )
        ok_message = Message.objects.create(
            mailbox=self.mailbox,
            subject='Whatever',
            from_header=self.arbitrary_email,
            body=arbitrary_body,
        )
        get_import_url_from_message_body.return_value = (
            arbitrary_url
        )
        get_source_from_user_and_url.return_value = (
            arbitrary_source
        )
        message_indicates_finish.return_value = (
            False
        )

        RunmeterConsumer.process_message(ok_message)

        RunmeterConsumer.get_import_url_from_message_body.assert_called_with(
            arbitrary_body,
        )
        RunmeterConsumer.get_source_from_user_and_url.assert_called_with(
            self.settings.user,
            arbitrary_url,
        )

    def test_get_route_name(self):
        document = self._get_sample_document()
        consumer = RunmeterConsumer(None)
        actual_value = consumer.get_route_name(document)
        expected_value = 'Cycle'

        self.assertEqual(
            actual_value,
            expected_value,
        )

    def test_get_start_time(self):
        document = self._get_sample_document()
        consumer = RunmeterConsumer(None)
        actual_value = consumer.get_start_time(document)
        expected_value = datetime.datetime(2013, 9, 6, 0, 51, 29).replace(
            tzinfo=utc
        )

        self.assertEqual(
            actual_value,
            expected_value,
        )

    def test_get_points(self):
        arbitrary_time = datetime.datetime.utcnow().replace(tzinfo=utc)
        document = self._get_sample_document()
        consumer = RunmeterConsumer(None)
        actual_value = consumer.get_points(document, arbitrary_time)

        with open(
            os.path.join(
                os.path.dirname(__file__),
                'files/expected_points.json',
            )
        ) as incoming:
            expected_value = json.loads(incoming.read())

        self.assertEqual(
            actual_value,
            expected_value,
        )

    def test_process_new_source(self):
        arbitrary_url = 'http://www.go.com/101'
        arbitrary_route_name = 'Something'
        arbitrary_source = models.LocationSource.objects.create(
            name='Whatnot',
            user=self.user,
            type=self.source_type,
            active=True,
            data={
                'url': arbitrary_url,
                'known_points': {},
            }
        )
        arbitrary_document = MagicMock()
        arbitrary_time = datetime.datetime.utcnow().replace(
            tzinfo=utc
        )
        arbitrary_points = [
            {'lat': -122, 'lng': 45, 'key': 'alpha', 'time': 1},
            {'lat': -123, 'lng': 44, 'key': 'beta', 'time': 2}
        ]

        consumer = RunmeterConsumer(arbitrary_source)
        consumer._get_document = MagicMock(
            return_value=arbitrary_document
        )
        consumer.get_start_time = MagicMock(
            return_value=arbitrary_time
        )
        consumer.get_route_name = MagicMock(
            return_value=arbitrary_route_name
        )
        consumer.get_points = MagicMock(
            return_value=arbitrary_points
        )
        consumer.is_active = MagicMock(
            return_value=False
        )

        consumer.process()

        consumer._get_document.assert_called_with(arbitrary_url)
        consumer.get_start_time.assert_called_with(arbitrary_document)
        consumer.get_route_name.assert_called_with(arbitrary_document)
        consumer.get_points.assert_called_with(
            arbitrary_document,
            arbitrary_time
        )

        actual_points = models.LocationSnapshot.objects.order_by('date')
        self.assertEqual(actual_points.count(), 2)

        first_assertions = {
            'date': arbitrary_time + datetime.timedelta(seconds=1),
            'source': arbitrary_source,
            'location': Point(-122, 45)
        }
        for k, v in first_assertions.items():
            self.assertEqual(getattr(actual_points[0], k), v)

        second_assertions = {
            'date': arbitrary_time + datetime.timedelta(seconds=2),
            'source': arbitrary_source,
            'location': Point(-123, 44)
        }
        for k, v in second_assertions.items():
            self.assertEqual(getattr(actual_points[1], k), v)

        self.assertFalse(
            models.LocationSource.objects.get(pk=arbitrary_source.pk).active
        )

    def test_process_existing_source(self):
        arbitrary_url = 'http://www.go.com/101'
        arbitrary_route_name = 'Something'
        arbitrary_source = models.LocationSource.objects.create(
            name='Whatnot',
            user=self.user,
            type=self.source_type,
            active=True,
            data={
                'url': arbitrary_url,
                'known_points': {
                    'alpha': 'arbitrary_value'
                },
            }
        )
        arbitrary_document = MagicMock()
        arbitrary_time = datetime.datetime.utcnow().replace(
            tzinfo=utc
        )
        arbitrary_points = [
            {'lat': -122, 'lng': 45, 'key': 'alpha', 'time': 1},
            {'lat': -123, 'lng': 44, 'key': 'beta', 'time': 2}
        ]

        consumer = RunmeterConsumer(arbitrary_source)
        consumer._get_document = MagicMock(
            return_value=arbitrary_document
        )
        consumer.get_start_time = MagicMock(
            return_value=arbitrary_time
        )
        consumer.get_route_name = MagicMock(
            return_value=arbitrary_route_name
        )
        consumer.get_points = MagicMock(
            return_value=arbitrary_points
        )
        consumer.is_active = MagicMock(
            return_value=True
        )

        consumer.process()

        consumer._get_document.assert_called_with(arbitrary_url)
        consumer.get_start_time.assert_called_with(arbitrary_document)
        consumer.get_route_name.assert_called_with(arbitrary_document)
        consumer.get_points.assert_called_with(
            arbitrary_document,
            arbitrary_time
        )

        actual_points = models.LocationSnapshot.objects.order_by('date')
        self.assertEqual(actual_points.count(), 1)

        assertions = {
            'date': arbitrary_time + datetime.timedelta(seconds=2),
            'source': arbitrary_source,
            'location': Point(-123, 44)
        }
        for k, v in assertions.items():
            self.assertEqual(getattr(actual_points[0], k), v)

        self.assertTrue(
            models.LocationSource.objects.get(pk=arbitrary_source.pk).active
        )

    def test_get_source_from_user_and_url_new(self):
        arbitrary_url = 'http://www.go.com/100'

        actual_source = RunmeterConsumer.get_source_from_user_and_url(
            self.user,
            arbitrary_url,
        )

        assertions = {
            'type': self.source_type,
            'data': {
                'url': arbitrary_url,
                'known_points': {},
            },
            'active': True
        }

        for k, v in assertions.items():
            self.assertEqual(getattr(actual_source, k), v)

    def test_get_source_from_user_and_url_existing(self):
        arbitrary_url = 'http://www.go.com/100'

        arbitrary_source = models.LocationSource.objects.create(
            name='Whatnot',
            user=self.user,
            type=self.source_type,
            data={
                'url': arbitrary_url,
                'known_points': {},
            }
        )

        actual_source = RunmeterConsumer.get_source_from_user_and_url(
            self.user,
            arbitrary_url,
        )

        self.assertEqual(
            arbitrary_source.pk,
            actual_source.pk,
        )

    def test_get_source_from_user_and_url_existing_too_old(self):
        arbitrary_url = 'http://www.go.com/100'

        arbitrary_source = models.LocationSource.objects.create(
            name='Whatnot',
            user=self.user,
            type=self.source_type,
            data={
                'url': arbitrary_url,
                'known_points': {},
            },
        )
        arbitrary_source.created = datetime.datetime(1970, 1, 1).replace(
            tzinfo=utc
        )
        arbitrary_source.save()

        actual_source = RunmeterConsumer.get_source_from_user_and_url(
            self.user,
            arbitrary_url,
        )

        self.assertNotEqual(
            arbitrary_source.pk,
            actual_source.pk,
        )

    @patch.object(RunmeterConsumer, 'process_message')
    def test_signal_receipt(self, process_message):
        arbitrary_body = 'OK'
        ok_message = Message.objects.create(
            mailbox=self.mailbox,
            subject='Whatever',
            from_header=self.arbitrary_email,
            body=arbitrary_body,
        )
        message_received.send(
            sender=self,
            message=ok_message
        )

        process_message.assert_called_with(ok_message)



import datetime
import json

from django.contrib.gis.geos import Point
import pytz
from social_auth.models import UserSocialAuth

from location.models import (
    LocationSnapshot,
    LocationSource,
    LocationSourceType
)
from location.signals import watch_location


class FoursquareConsumer(object):
    def __init__(self, data):
        self.data = json.loads(data)

    def process_checkin(self):
        if self.data['type'] == 'checkin':
            user = self.get_user()
            source = LocationSource.objects.create(
                name=self.data['venue']['name'],
                user=user,
                type=self.get_source_type(),
                data=self.data
            )

            with watch_location(user):
                return LocationSnapshot.objects.create(
                    location=Point(
                        self.data['venue']['location']['lng'],
                        self.data['venue']['location']['lat'],
                    ),
                    date=(
                        datetime.datetime.fromtimestamp(
                            self.data['createdAt'],
                            pytz.timezone(self.data['timeZone']),
                        )
                    ),
                    source=source,
                )
        return None

    def get_user(self):
        return UserSocialAuth.objects.get(
            uid=self.data['user']['id'],
            provider='foursquare',
        ).user

    @classmethod
    def get_source_type(cls):
        source_type, _ = LocationSourceType.objects.get_or_create(
            name='Foursquare Check-in'
        )
        return source_type
